@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile: jobsingletransfer.c,v $
-                             -------------------
-    cvs         : $Id: jobsingletransfer.c,v 1.1 2005/08/08 02:24:41 aquamaniac Exp $
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2004 by Martin Preuss
+    copyright   : (C) 2004-2011 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -29,6 +26,7 @@
 #include <aqbanking/jobsingletransfer.h>
 #include <aqbanking/jobsingletransfer_be.h>
 #include <aqbanking/jobsingledebitnote_be.h>
+#include <aqbanking/jobinternaltransfer_be.h>
 #include <aqbanking/jobcreatesto_be.h>
 #include <aqbanking/jobmodifysto_be.h>
 #include <aqbanking/jobdeletesto_be.h>
@@ -49,78 +47,114 @@ GWEN_INHERIT(AH_JOB, AH_JOB_SINGLETRANSFER);
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_SingleTransfer_new(AH_CUSTOMER *cu,
-                                  AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account, AB_Job_TypeTransfer);
+AH_JOB *AH_Job_SingleTransfer_new(AB_USER *u,
+				  AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account, AB_Job_TypeTransfer);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_SingleDebitNote_new(AH_CUSTOMER *cu,
-                                   AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account, AB_Job_TypeDebitNote);
+AH_JOB *AH_Job_SingleDebitNote_new(AB_USER *u,
+                                   AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account, AB_Job_TypeDebitNote);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_CreateStandingOrder_new(AH_CUSTOMER *cu,
-                                       AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account,
-                                       AB_Job_TypeCreateStandingOrder);
+AH_JOB *AH_Job_CreateStandingOrder_new(AB_USER *u,
+                                       AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account,
+				  AB_Job_TypeCreateStandingOrder);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_ModifyStandingOrder_new(AH_CUSTOMER *cu,
-                                       AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account,
-                                       AB_Job_TypeModifyStandingOrder);
+AH_JOB *AH_Job_ModifyStandingOrder_new(AB_USER *u,
+                                       AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account,
+				  AB_Job_TypeModifyStandingOrder);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_DeleteStandingOrder_new(AH_CUSTOMER *cu,
-                                       AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account,
-                                       AB_Job_TypeDeleteStandingOrder);
+AH_JOB *AH_Job_DeleteStandingOrder_new(AB_USER *u,
+                                       AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account,
+				  AB_Job_TypeDeleteStandingOrder);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_CreateDatedTransfer_new(AH_CUSTOMER *cu,
-                                       AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account,
-                                       AB_Job_TypeCreateDatedTransfer);
+AH_JOB *AH_Job_CreateDatedTransfer_new(AB_USER *u,
+                                       AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account,
+				  AB_Job_TypeCreateDatedTransfer);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_ModifyDatedTransfer_new(AH_CUSTOMER *cu,
-                                       AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account,
-                                       AB_Job_TypeModifyDatedTransfer);
+AH_JOB *AH_Job_ModifyDatedTransfer_new(AB_USER *u,
+                                       AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account,
+				  AB_Job_TypeModifyDatedTransfer);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_DeleteDatedTransfer_new(AH_CUSTOMER *cu,
-                                       AH_ACCOUNT *account) {
-  return AH_Job_SingleTransferBase_new(cu, account,
-                                       AB_Job_TypeDeleteDatedTransfer);
+AH_JOB *AH_Job_DeleteDatedTransfer_new(AB_USER *u,
+                                       AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account,
+				  AB_Job_TypeDeleteDatedTransfer);
+  return j;
 }
 
 
 
 /* --------------------------------------------------------------- FUNCTION */
-AH_JOB *AH_Job_SingleTransferBase_new(AH_CUSTOMER *cu,
-                                      AH_ACCOUNT *account,
+AH_JOB *AH_Job_InternalTransfer_new(AB_USER *u,
+                                    AB_ACCOUNT *account) {
+  AH_JOB *j;
+
+  j=AH_Job_SingleTransferBase_new(u, account,
+				  AB_Job_TypeInternalTransfer);
+  return j;
+}
+
+
+
+/* --------------------------------------------------------------- FUNCTION */
+AH_JOB *AH_Job_SingleTransferBase_new(AB_USER *u,
+                                      AB_ACCOUNT *account,
                                       AB_JOB_TYPE jobType) {
   AH_JOB *j;
   AH_JOB_SINGLETRANSFER *aj;
@@ -128,28 +162,31 @@ AH_JOB *AH_Job_SingleTransferBase_new(AH_CUSTOMER *cu,
 
   switch(jobType) {
   case AB_Job_TypeTransfer:
-    j=AH_AccountJob_new("JobSingleTransfer", cu, account);
+    j=AH_AccountJob_new("JobSingleTransfer", u, account);
     break;
   case AB_Job_TypeDebitNote:
-    j=AH_AccountJob_new("JobSingleDebitNote", cu, account);
+    j=AH_AccountJob_new("JobSingleDebitNote", u, account);
     break;
   case AB_Job_TypeCreateStandingOrder:
-    j=AH_AccountJob_new("JobCreateStandingOrder", cu, account);
+    j=AH_AccountJob_new("JobCreateStandingOrder", u, account);
     break;
   case AB_Job_TypeModifyStandingOrder:
-    j=AH_AccountJob_new("JobModifyStandingOrder", cu, account);
+    j=AH_AccountJob_new("JobModifyStandingOrder", u, account);
     break;
   case AB_Job_TypeDeleteStandingOrder:
-    j=AH_AccountJob_new("JobDeleteStandingOrder", cu, account);
+    j=AH_AccountJob_new("JobDeleteStandingOrder", u, account);
     break;
   case AB_Job_TypeCreateDatedTransfer:
-    j=AH_AccountJob_new("JobCreateDatedTransfer", cu, account);
+    j=AH_AccountJob_new("JobCreateDatedTransfer", u, account);
     break;
   case AB_Job_TypeModifyDatedTransfer:
-    j=AH_AccountJob_new("JobModifyDatedTransfer", cu, account);
+    j=AH_AccountJob_new("JobModifyDatedTransfer", u, account);
     break;
   case AB_Job_TypeDeleteDatedTransfer:
-    j=AH_AccountJob_new("JobDeleteDatedTransfer", cu, account);
+    j=AH_AccountJob_new("JobDeleteDatedTransfer", u, account);
+    break;
+  case AB_Job_TypeInternalTransfer:
+    j=AH_AccountJob_new("JobInternalTransfer", u, account);
     break;
   default:
     DBG_ERROR(AQHBCI_LOGDOMAIN, "Unknown job type %d", jobType);
@@ -166,6 +203,7 @@ AH_JOB *AH_Job_SingleTransferBase_new(AH_CUSTOMER *cu,
   /* overwrite some virtual functions */
   AH_Job_SetProcessFn(j, AH_Job_SingleTransfer_Process);
   AH_Job_SetExchangeFn(j, AH_Job_SingleTransfer_Exchange);
+  AH_Job_SetAddChallengeParamsFn(j, AH_Job_SingleTransfer_AddChallengeParams);
 
   /* set some known arguments */
   dbArgs=AH_Job_GetArguments(j);
@@ -178,13 +216,15 @@ AH_JOB *AH_Job_SingleTransferBase_new(AH_CUSTOMER *cu,
 
 
 /* --------------------------------------------------------------- FUNCTION */
-void AH_Job_SingleTransfer_FreeData(void *bp, void *p){
+void GWENHYWFAR_CB AH_Job_SingleTransfer_FreeData(void *bp, void *p){
   AH_JOB_SINGLETRANSFER *aj;
 
   aj=(AH_JOB_SINGLETRANSFER*)p;
   free(aj->fiid);
   free(aj->oldFiid);
 
+  AB_Transaction_free(aj->validatedTransaction);
+
   GWEN_FREE_OBJECT(aj);
 }
 
@@ -217,7 +257,7 @@ const char *AH_Job_SingleTransfer_GetOldFiid(AH_JOB *j) {
 
 
 /* --------------------------------------------------------------- FUNCTION */
-int AH_Job_SingleTransfer_Process(AH_JOB *j){
+int AH_Job_SingleTransfer_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx){
   AH_JOB_SINGLETRANSFER *aj;
   GWEN_DB_NODE *dbResponses;
   GWEN_DB_NODE *dbCurr;
@@ -235,6 +275,7 @@ int AH_Job_SingleTransfer_Process(AH_JOB *j){
   case AB_Job_TypeDebitNote:
   case AB_Job_TypeDeleteStandingOrder:
   case AB_Job_TypeDeleteDatedTransfer:
+  case AB_Job_TypeInternalTransfer:
     dbCurr=GWEN_DB_GetFirstGroup(dbResponses);
     while(dbCurr) {
       int rv;
@@ -457,26 +498,22 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
   case AB_Job_TypeDeleteDatedTransfer:
     lim=AB_JobDeleteDatedTransfer_GetFieldLimits(bj);
     break;
+  case AB_Job_TypeInternalTransfer:
+    lim=AB_JobInternalTransfer_GetFieldLimits(bj);
+    break;
   default:
     DBG_ERROR(AQHBCI_LOGDOMAIN,
               "Unhandled job type %d", aj->jobType);
     return -1;
   }
 
-  /* collapse splits if any */
-  n=AB_Split_List_GetCount(AB_Transaction_GetSplits(t));
-  if (n) {
-    DBG_ERROR(AQHBCI_LOGDOMAIN,
-	      "Transfer contains splits, unable to handle");
-    return -1;
-  }
-
   /* check purpose */
   if (lim) {
     maxn=AB_TransactionLimits_GetMaxLinesPurpose(lim);
     maxs=AB_TransactionLimits_GetMaxLenPurpose(lim);
   }
   else {
+    DBG_INFO(AQHBCI_LOGDOMAIN, "No transaction limits");
     maxn=0;
     maxs=0;
   }
@@ -500,19 +537,28 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
 	if (maxn && n>maxn) {
 	  DBG_ERROR(AQHBCI_LOGDOMAIN,
 		    "Too many purpose lines (%d>%d)", n, maxn);
+	  GWEN_Gui_ProgressLog2(0,
+				GWEN_LoggerLevel_Error,
+				I18N("Too many purpose lines (%d>%d)"),
+				n, maxn);
 	  GWEN_StringList_free(nsl);
-	  return AB_ERROR_INVALID;
+	  return GWEN_ERROR_INVALID;
 	}
 	tbuf=GWEN_Buffer_new(0, maxs, 0, 1);
 	AB_ImExporter_Utf8ToDta(p, -1, tbuf);
+	GWEN_Text_CondenseBuffer(tbuf);
 	l=GWEN_Buffer_GetUsedBytes(tbuf);
-	if (l>maxs) {
+	if (maxs && l>maxs) {
 	  DBG_ERROR(AQHBCI_LOGDOMAIN,
 		    "Too many chars in purpose line %d (%d>%d)", n, l, maxs);
+	  GWEN_Gui_ProgressLog2(0,
+				GWEN_LoggerLevel_Error,
+				I18N("Too many chars in purpose line %d (%d>%d)"),
+				n, l, maxs);
 	  GWEN_StringList_free(nsl);
-	  return AB_ERROR_INVALID;
+	  return GWEN_ERROR_INVALID;
 	}
-	np=(char*)malloc(l+1);
+	np=(char*)GWEN_Memory_malloc(l+1);
 	memmove(np, GWEN_Buffer_GetStart(tbuf), l+1);
 	GWEN_Buffer_free(tbuf);
 	/* let string list take the newly alllocated string */
@@ -524,7 +570,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
   }
   if (!n) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "No purpose lines");
-    return AB_ERROR_INVALID;
+    return GWEN_ERROR_INVALID;
   }
 
   /* check remote name */
@@ -558,19 +604,20 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
 		    "Too many remote name lines (%d>%d)",
 		    n, maxn);
           GWEN_StringList_free(nsl);
-	  return AB_ERROR_INVALID;
+	  return GWEN_ERROR_INVALID;
 	}
 	tbuf=GWEN_Buffer_new(0, maxs, 0, 1);
         AB_ImExporter_Utf8ToDta(p, -1, tbuf);
+	GWEN_Text_CondenseBuffer(tbuf);
 	l=GWEN_Buffer_GetUsedBytes(tbuf);
 	if (l>maxs) {
-	  DBG_WARN(AQHBCI_LOGDOMAIN,
+	  DBG_ERROR(AQHBCI_LOGDOMAIN,
 		   "Too many chars in remote name line %d (%d>%d)",
 		   n, l, maxs);
           GWEN_StringList_free(nsl);
-	  return AB_ERROR_INVALID;
+	  return GWEN_ERROR_INVALID;
 	}
-	np=(char*)malloc(l+1);
+	np=(char*)GWEN_Memory_malloc(l+1);
 	memmove(np, GWEN_Buffer_GetStart(tbuf), l+1);
 	GWEN_Buffer_free(tbuf);
 	/* let string list take the newly alllocated string */
@@ -582,7 +629,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
   }
   if (!n) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "No remote name lines");
-    return AB_ERROR_INVALID;
+    return GWEN_ERROR_INVALID;
   }
 
   /* check local name */
@@ -590,7 +637,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
   if (!s) {
     AB_ACCOUNT *a;
 
-    DBG_WARN(AQHBCI_LOGDOMAIN,
+    DBG_NOTICE(AQHBCI_LOGDOMAIN,
 	     "No local name, filling in");
     a=AB_Job_GetAccount(bj);
     assert(a);
@@ -598,7 +645,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
     if (!s) {
       DBG_ERROR(AQHBCI_LOGDOMAIN,
 		"No owner name in account, giving up");
-      return AB_ERROR_INVALID;
+      return GWEN_ERROR_INVALID;
     }
     AB_Transaction_SetLocalName(t, s);
   }
@@ -606,13 +653,13 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
   /* check local bank code */
   s=AB_Transaction_GetLocalBankCode(t);
   if (!s) {
-    AH_ACCOUNT *a;
+    AB_ACCOUNT *a;
 
     DBG_WARN(AQHBCI_LOGDOMAIN,
 	     "No local bank code, filling in");
     a=AH_AccountJob_GetAccount(mj);
     assert(a);
-    s=AH_Account_GetBankId(a);
+    s=AB_Account_GetBankCode(a);
     assert(s);
     AB_Transaction_SetLocalBankCode(t, s);
   }
@@ -620,13 +667,13 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
   /* check local account number */
   s=AB_Transaction_GetLocalAccountNumber(t);
   if (!s) {
-    AH_ACCOUNT *a;
+    AB_ACCOUNT *a;
 
     DBG_WARN(AQHBCI_LOGDOMAIN,
 	     "No local account number, filling in");
     a=AH_AccountJob_GetAccount(mj);
     assert(a);
-    s=AH_Account_GetAccountId(a);
+    s=AB_Account_GetAccountNumber(a);
     assert(s);
     AB_Transaction_SetLocalAccountNumber(t, s);
   }
@@ -634,14 +681,14 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
   /* check local account suffix */
   s=AB_Transaction_GetLocalSuffix(t);
   if (!s) {
-    AH_ACCOUNT *a;
+    AB_ACCOUNT *a;
 
     DBG_INFO(AQHBCI_LOGDOMAIN,
 	     "No local suffix, filling in (if possible)");
     a=AH_AccountJob_GetAccount(mj);
     assert(a);
-    s=AH_Account_GetSuffix(a);
-    if (s)
+    s=AB_Account_GetSubAccountId(a);
+    if (s && *s)
       AB_Transaction_SetLocalSuffix(t, s);
   }
 
@@ -660,6 +707,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
         case AB_Job_TypeCreateStandingOrder:
         case AB_Job_TypeModifyStandingOrder:
         case AB_Job_TypeDeleteStandingOrder:
+        case AB_Job_TypeInternalTransfer:
         default:
           n=51; /* "Ueberweisung" */
           break;
@@ -671,7 +719,11 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
       if (!AB_TransactionLimits_HasValuesTextKey(lim, numbuf)) {
 	DBG_ERROR(AQHBCI_LOGDOMAIN, "Text key \"%s\" not supported by bank",
 		  numbuf);
-	return AB_ERROR_INVALID;
+	GWEN_Gui_ProgressLog2(0,
+			      GWEN_LoggerLevel_Error,
+			      I18N("Text key \"%d\" not supported by the bank"),
+			      n);
+	return GWEN_ERROR_INVALID;
       }
     }
   }
@@ -698,7 +750,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
           if (n==0) {
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "No cycle given");
-            return AB_ERROR_INVALID;
+            return GWEN_ERROR_INVALID;
           }
 
           snprintf(numbuf, sizeof(numbuf), "%d", n);
@@ -707,7 +759,11 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "Month day \"%s\" not supported by bank",
                       numbuf);
-            return AB_ERROR_INVALID;
+	    GWEN_Gui_ProgressLog2(0,
+				  GWEN_LoggerLevel_Error,
+				  I18N("Month day \"%d\" not supported by bank"),
+				  n);
+            return GWEN_ERROR_INVALID;
           }
         }
 
@@ -720,7 +776,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
           if (n==0) {
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "No execution day given");
-            return AB_ERROR_INVALID;
+            return GWEN_ERROR_INVALID;
           }
 
           snprintf(numbuf, sizeof(numbuf), "%d", n);
@@ -729,7 +785,11 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "Execution month day \"%s\" not supported by bank",
                       numbuf);
-            return AB_ERROR_INVALID;
+	    GWEN_Gui_ProgressLog2(0,
+				  GWEN_LoggerLevel_Error,
+				  I18N("Execution month day \"%d\" not supported by bank"),
+				  n);
+            return GWEN_ERROR_INVALID;
           }
         } /* if there are limits */
       }
@@ -745,7 +805,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
           if (n==0) {
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "No cycle given");
-            return AB_ERROR_INVALID;
+            return GWEN_ERROR_INVALID;
           }
 
           snprintf(numbuf, sizeof(numbuf), "%d", n);
@@ -754,7 +814,11 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "Week day \"%s\" not supported by bank",
                       numbuf);
-            return AB_ERROR_INVALID;
+	    GWEN_Gui_ProgressLog2(0,
+				  GWEN_LoggerLevel_Error,
+				  I18N("Week day \"%d\" not supported by bank"),
+				  n);
+            return GWEN_ERROR_INVALID;
           }
         } /* if there are limits */
 
@@ -767,7 +831,7 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
           if (n==0) {
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "No execution day given");
-            return AB_ERROR_INVALID;
+            return GWEN_ERROR_INVALID;
           }
 
           snprintf(numbuf, sizeof(numbuf), "%d", n);
@@ -776,14 +840,18 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
             DBG_ERROR(AQHBCI_LOGDOMAIN,
                       "Execution month day \"%s\" not supported by bank",
                       numbuf);
-            return AB_ERROR_INVALID;
+	    GWEN_Gui_ProgressLog2(0,
+				  GWEN_LoggerLevel_Error,
+				  I18N("Execution month day \"%d\" not supported by bank"),
+				  n);
+            return GWEN_ERROR_INVALID;
           }
         } /* if there are limits */
       }
       else {
         DBG_ERROR(AQHBCI_LOGDOMAIN,
                   "Unsupported period %d", AB_Transaction_GetPeriod(t));
-        return AB_ERROR_INVALID;
+        return GWEN_ERROR_INVALID;
       }
 
       /* check setup times */
@@ -803,7 +871,12 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
         if (n && dt<n) {
           DBG_ERROR(AQHBCI_LOGDOMAIN,
                     "Minimum setup time violated");
-          return AB_ERROR_INVALID;
+	  GWEN_Gui_ProgressLog2(0,
+				GWEN_LoggerLevel_Error,
+				I18N("Minimum setup time violated. "
+				     "Dated transactions need to be at least %d days away"),
+				n);
+          return GWEN_ERROR_INVALID;
         }
 
         /* check maximum setup time */
@@ -811,7 +884,12 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
         if (n && dt>n) {
           DBG_ERROR(AQHBCI_LOGDOMAIN,
                     "Maximum setup time violated");
-          return AB_ERROR_INVALID;
+	  GWEN_Gui_ProgressLog2(0,
+				GWEN_LoggerLevel_Error,
+				I18N("Maximum setup time violated. "
+				     "Dated transactions need to be at most %d days away"),
+				n);
+          return GWEN_ERROR_INVALID;
         }
       }
       break;
@@ -834,7 +912,12 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
         if (n && dt<n) {
           DBG_ERROR(AQHBCI_LOGDOMAIN,
                     "Minimum setup time violated");
-          return AB_ERROR_INVALID;
+	  GWEN_Gui_ProgressLog2(0,
+				GWEN_LoggerLevel_Error,
+				I18N("Minimum setup time violated. "
+				     "Dated transactions need to be at least %d days away"),
+				n);
+          return GWEN_ERROR_INVALID;
         }
 
         /* check maximum setup time */
@@ -842,7 +925,12 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
         if (n && dt>n) {
           DBG_ERROR(AQHBCI_LOGDOMAIN,
                     "Maximum setup time violated");
-          return AB_ERROR_INVALID;
+	  GWEN_Gui_ProgressLog2(0,
+				GWEN_LoggerLevel_Error,
+				I18N("Maximum setup time violated. "
+				     "Dated transactions need to be at most %d days away"),
+				n);
+          return GWEN_ERROR_INVALID;
         }
       }
       break;
@@ -861,9 +949,228 @@ int AH_Job_SingleTransfer__ValidateTransfer(AB_JOB *bj,
 
 
 
+int AH_Job_SingleTransfer_AddChallengeParams(AH_JOB *j, int hkTanVer, GWEN_DB_NODE *dbMethod) {
+  AH_JOB_SINGLETRANSFER *aj;
+  const AB_TRANSACTION *t;
+  const char *s;
+  int tanVer=AH_JOB_TANVER_1_4;
+
+  DBG_ERROR(AQHBCI_LOGDOMAIN, "AddChallengeParams function called");
+
+  assert(j);
+  aj=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_SINGLETRANSFER, j);
+  assert(aj);
+
+  t=aj->validatedTransaction;
+  if (t==NULL) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "No validated transaction");
+    return GWEN_ERROR_INVALID;
+  }
+
+  s=GWEN_DB_GetCharValue(dbMethod, "zkaTanVersion", 0, NULL);
+  if (s && *s && strncasecmp(s, "1.3", 3)==0) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "TAN version is 1.3 (%s)", s);
+    tanVer=AH_JOB_TANVER_1_3;
+  }
+
+  if (tanVer==AH_JOB_TANVER_1_4) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "TAN version is 1.4.x");
+
+    switch(aj->jobType) {
+    case AB_Job_TypeTransfer:
+    case AB_Job_TypeDebitNote:
+    case AB_Job_TypeCreateStandingOrder:
+    case AB_Job_TypeModifyStandingOrder:
+    case AB_Job_TypeDeleteStandingOrder:
+      {
+        const char *s;
+        const AB_VALUE *v;
+
+        /* select challenge class (why the heck doesn't the bank derive this from the job??) */
+        switch(aj->jobType) {
+        case AB_Job_TypeTransfer:
+          AH_Job_SetChallengeClass(j, 4);
+          break;
+        case AB_Job_TypeDebitNote:
+          AH_Job_SetChallengeClass(j, 15);
+          break;
+        case AB_Job_TypeCreateStandingOrder:
+        case AB_Job_TypeModifyStandingOrder:
+        case AB_Job_TypeDeleteStandingOrder:
+          AH_Job_SetChallengeClass(j, 34);
+          break;
+        default:
+          /* TSNH */
+          break;
+        }
+
+        /* P1: Betrag */
+        v=AB_Transaction_GetValue(t);
+        if (v) {
+          GWEN_BUFFER *tbuf;
+  
+          tbuf=GWEN_Buffer_new(0, 64, 0, 1);
+          AH_Job_ValueToChallengeString(v, tbuf);
+          AH_Job_AddChallengeParam(j, GWEN_Buffer_GetStart(tbuf));
+          GWEN_Buffer_free(tbuf);
+        }
+  
+        /* P2: BLZ Zahler/Empfaenger */
+        s=AB_Transaction_GetRemoteBankCode(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P3: Konto Zahler/Empfaenger */
+        s=AB_Transaction_GetRemoteAccountNumber(t);
+        if (s && *s) {
+          int i;
+          GWEN_BUFFER *tbuf;
+
+          tbuf=GWEN_Buffer_new(0, 64, 0, 1);
+          GWEN_Buffer_AppendString(tbuf, s);
+          i=10-strlen(s);
+          if (i>0) {
+            /* need to left-fill the account number with leading zeroes
+             * to a length of exactly 10 digits */
+            GWEN_Buffer_Rewind(tbuf);
+            GWEN_Buffer_FillLeftWithBytes(tbuf, '0', i);
+          }
+          AH_Job_AddChallengeParam(j, GWEN_Buffer_GetStart(tbuf));
+          GWEN_Buffer_free(tbuf);
+        }
+        else {
+          DBG_ERROR(AQHBCI_LOGDOMAIN, "No account number");
+          return GWEN_ERROR_INVALID;
+        }
+
+#if 0 /* don't provide more info than needed, more data=more trouble sources */
+        /* P4: BLZ Absender (O) */
+        s=AB_Transaction_GetLocalBankCode(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P5: Konto Absender (O) */
+        s=AB_Transaction_GetLocalAccountNumber(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        if (aj->jobType==AB_Job_TypeTransfer) {
+          /* P6: Name Absender (O) */
+          s=AB_Transaction_GetLocalName(t);
+          assert(s && *s);
+          AH_Job_AddChallengeParam(j, s);
+        }
+#endif
+
+      }
+      break;
+  
+    case AB_Job_TypeCreateDatedTransfer:
+    case AB_Job_TypeModifyDatedTransfer:
+    case AB_Job_TypeDeleteDatedTransfer:
+      {
+        const char *s;
+        const AB_VALUE *v;
+        const GWEN_TIME *ti;
+        GWEN_BUFFER *tbuf;
+  
+        /* P1: Betrag */
+        v=AB_Transaction_GetValue(t);
+        assert(v);
+        tbuf=GWEN_Buffer_new(0, 64, 0, 1);
+        AB_Value_toHumanReadableString2(v, tbuf, 2, 0); /* TODO: currency needed?? */
+        AH_Job_AddChallengeParam(j, GWEN_Buffer_GetStart(tbuf));
+        GWEN_Buffer_free(tbuf);
+  
+        /* P2: BLZ Empfaenger */
+        s=AB_Transaction_GetRemoteBankCode(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P3: Konto Empfaenger */
+        s=AB_Transaction_GetRemoteAccountNumber(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P4: Termin (M) */
+        ti=AB_Transaction_GetDate(t);
+        assert(ti);
+  
+        tbuf=GWEN_Buffer_new(0, 64, 0, 1);
+        GWEN_Time_toString(ti, "YYYYMMDD", tbuf);
+        AH_Job_AddChallengeParam(j, GWEN_Buffer_GetStart(tbuf));
+        GWEN_Buffer_free(tbuf);
+  
+        /* P5: BLZ Absender (O) */
+        s=AB_Transaction_GetLocalBankCode(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P6: Konto Absender (O) */
+        s=AB_Transaction_GetLocalAccountNumber(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P7: Name Absender (O) */
+        s=AB_Transaction_GetLocalName(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+      }
+      break;
+  
+    case AB_Job_TypeInternalTransfer:
+      {
+        const char *s;
+        const AB_VALUE *v;
+        GWEN_BUFFER *tbuf;
+  
+        AH_Job_SetChallengeClass(j, 5);
+  
+        /* P1: Betrag */
+        v=AB_Transaction_GetValue(t);
+        assert(v);
+  
+        tbuf=GWEN_Buffer_new(0, 64, 0, 1);
+        AB_Value_toHumanReadableString2(v, tbuf, 2, 0); /* TODO: currency needed?? */
+        AH_Job_AddChallengeParam(j, GWEN_Buffer_GetStart(tbuf));
+        GWEN_Buffer_free(tbuf);
+  
+        /* P2: Konto Empfaenger */
+        s=AB_Transaction_GetRemoteAccountNumber(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P3: Konto Absender (O) */
+        s=AB_Transaction_GetLocalAccountNumber(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+  
+        /* P4: Name Empfaenger (O) (we use the local name here instead, but it is the same as the remote name) */
+        s=AB_Transaction_GetLocalName(t);
+        assert(s && *s);
+        AH_Job_AddChallengeParam(j, s);
+      }
+      break;
+  
+    default:
+      DBG_ERROR(AQHBCI_LOGDOMAIN,
+                "Unhandled job type %d", aj->jobType);
+      return GWEN_ERROR_INVALID;
+    } /* switch */
+  }
+  else {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "Unhandled tan version %d for now", tanVer);
+    return GWEN_ERROR_INTERNAL;
+  }
+  return 0;
+}
+
+
+
 /* --------------------------------------------------------------- FUNCTION */
 int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
-				   AH_JOB_EXCHANGE_MODE m){
+				   AH_JOB_EXCHANGE_MODE m,
+				   AB_IMEXPORTER_CONTEXT *ctx){
   AH_JOB_SINGLETRANSFER *aj;
 
   DBG_INFO(AQHBCI_LOGDOMAIN, "Exchanging (%d)", m);
@@ -875,7 +1182,7 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
   if (aj->jobType!=AB_Job_GetType(bj)) {
     DBG_ERROR(AQHBCI_LOGDOMAIN,
               "Different job types");
-    return AB_ERROR_INVALID;
+    return GWEN_ERROR_INVALID;
   }
 
   switch(m) {
@@ -888,8 +1195,8 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 
     dbParams=AH_Job_GetParams(j);
     DBG_DEBUG(AQHBCI_LOGDOMAIN, "Have this parameters to exchange:");
-    if (GWEN_Logger_GetLevel(AQHBCI_LOGDOMAIN)>=GWEN_LoggerLevelDebug)
-      GWEN_DB_Dump(dbParams, stderr, 2);
+    if (GWEN_Logger_GetLevel(AQHBCI_LOGDOMAIN)>=GWEN_LoggerLevel_Debug)
+      GWEN_DB_Dump(dbParams, 2);
 
     /* read limits */
     lim=AB_TransactionLimits_new();
@@ -986,16 +1293,16 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
       s=GWEN_DB_GetCharValue(dbParams, "AllowedWeekDays", 0, 0);
       if (s && *s) {
 	while(*s) {
-	  char buf[3];
+	  char buf[2];
           const char *x;
 
-          buf[2]=0;
-          strncpy(buf, s, 2);
+	  buf[0]=*s;
+	  buf[1]=0;
           x=buf;
           if (*x=='0')
             x++;
           AB_TransactionLimits_AddValuesExecutionDayWeek(lim, x, 0);
-	  s+=2;
+	  s++;
 	} /* while */
         GWEN_StringList_Sort(AB_TransactionLimits_GetValuesExecutionDayWeek(lim),
                              1, GWEN_StringList_SortModeInt);
@@ -1107,12 +1414,16 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
     case AB_Job_TypeDeleteDatedTransfer:
       AB_JobDeleteDatedTransfer_SetFieldLimits(bj, lim);
       break;
+    case AB_Job_TypeInternalTransfer:
+      AB_JobInternalTransfer_SetFieldLimits(bj, lim);
+      break;
     default:
       DBG_ERROR(AQHBCI_LOGDOMAIN,
                 "Unhandled job type %d", aj->jobType);
       return -1;
     }
 
+    AB_TransactionLimits_free(lim);
     return 0;
   }
 
@@ -1148,10 +1459,13 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
     case AB_Job_TypeDeleteDatedTransfer:
       ot=AB_JobDeleteDatedTransfer_GetTransaction(bj);
       break;
+    case AB_Job_TypeInternalTransfer:
+      ot=AB_JobInternalTransfer_GetTransaction(bj);
+      break;
     default:
       DBG_ERROR(AQHBCI_LOGDOMAIN,
                 "Unhandled job type %d", aj->jobType);
-      return AB_ERROR_INVALID;
+      return GWEN_ERROR_INVALID;
     }
 
     if (ot) {
@@ -1167,7 +1481,7 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 	DBG_ERROR(AQHBCI_LOGDOMAIN,
 		  "Invalid transaction");
 	AB_Job_SetStatus(bj, AB_Job_StatusError);
-	return AB_ERROR_INVALID;
+	return GWEN_ERROR_INVALID;
       }
       /* store the validated transaction back into application job,
        * to allow the application to recognize answers to this job later */
@@ -1196,12 +1510,19 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
       case AB_Job_TypeDeleteDatedTransfer:
         AB_JobDeleteDatedTransfer_SetTransaction(bj, t);
         break;
+      case AB_Job_TypeInternalTransfer:
+        AB_JobInternalTransfer_SetTransaction(bj, t);
+        break;
       default:
         DBG_ERROR(AQHBCI_LOGDOMAIN,
                   "Unhandled job type %d", aj->jobType);
-        return AB_ERROR_INVALID;
+        return GWEN_ERROR_INVALID;
       }
 
+      /* preset challenge stuff */
+      AH_Job_SetChallengeClass(j, 4);
+      AH_Job_SetChallengeValue(j, AB_Transaction_GetValue(t));
+
       dbT=GWEN_DB_GetGroup(dbArgs, GWEN_DB_FLAGS_OVERWRITE_GROUPS,
                            "transaction");
       assert(dbT);
@@ -1261,7 +1582,7 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
         assert(dbV);
 
 	nbuf=GWEN_Buffer_new(0, 32, 0, 1);
-	if (GWEN_Text_DoubleToBuffer(AB_Value_GetValue(v),
+	if (GWEN_Text_DoubleToBuffer(AB_Value_GetValueAsDouble(v),
 				     nbuf)) {
 	  DBG_ERROR(AQHBCI_LOGDOMAIN, "Buffer overflow");
           GWEN_Buffer_free(nbuf);
@@ -1400,7 +1721,7 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
         default:
           DBG_ERROR(AQHBCI_LOGDOMAIN, "Unsupported period %d",
                     AB_Transaction_GetPeriod(t));
-          return AB_ERROR_INVALID;
+          return GWEN_ERROR_INVALID;
         }
         GWEN_DB_SetCharValue(dbT,
                              GWEN_DB_FLAGS_OVERWRITE_VARS,
@@ -1457,11 +1778,14 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
         break;
       } /* switch */
 
+      /* store pointer to the validated transaction */
+      AB_Transaction_Attach(t);
+      aj->validatedTransaction=t;
     }
     else {
       DBG_ERROR(AQHBCI_LOGDOMAIN, "No transaction");
       AB_Job_SetStatus(bj, AB_Job_StatusError);
-      return AB_ERROR_NO_DATA;
+      return GWEN_ERROR_NO_DATA;
     }
     return 0;
   }
@@ -1472,6 +1796,7 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
     int has10;
     int has20;
     const AB_TRANSACTION *ot;
+    AB_TRANSACTION_STATUS tStatus;
 
     rl=AH_Job_GetSegResults(j);
     assert(rl);
@@ -1480,16 +1805,19 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
     if (!r) {
       DBG_ERROR(AQHBCI_LOGDOMAIN, "No segment results");
       AB_Job_SetStatus(bj, AB_Job_StatusError);
-      /* return AB_ERROR_NO_DATA; */
+      /* return GWEN_ERROR_NO_DATA; */
       return 0;
     }
     has10=0;
     has20=0;
     while(r) {
-      if (AH_Result_GetCode(r)==10)
-        has10=1;
-      else if (AH_Result_GetCode(r)==20)
-        has20=1;
+      int rcode;
+
+      rcode=AH_Result_GetCode(r);
+      if (rcode <=19)
+	has10=1;
+      else if (rcode>=20 && rcode <=29)
+	has20=1;
       r=AH_Result_List_Next(r);
     }
 
@@ -1501,6 +1829,7 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
       switch(aj->jobType) {
       case AB_Job_TypeTransfer:
       case AB_Job_TypeDebitNote:
+      case AB_Job_TypeInternalTransfer:
 	AB_Job_SetStatus(bj, AB_Job_StatusPending);
 	DBG_INFO(AQHBCI_LOGDOMAIN, "Job pending");
 	break;
@@ -1524,9 +1853,47 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
       AB_Job_SetStatus(bj, AB_Job_StatusError);
     }
 
+    if (has20)
+      tStatus=AB_Transaction_StatusAccepted;
+    else if (has10)
+      tStatus=AB_Transaction_StatusPending;
+    else
+      tStatus=AB_Transaction_StatusRejected;
+
     switch(aj->jobType) {
     case AB_Job_TypeTransfer:
+      ot=AB_JobSingleTransfer_GetTransaction(bj);
+      if (ot) {
+	AB_TRANSACTION *t;
+
+	t=AB_Transaction_dup(ot);
+	AB_Transaction_SetStatus(t, tStatus);
+	AB_Transaction_SetType(t, AB_Transaction_TypeTransfer);
+	AB_ImExporterContext_AddTransfer(ctx, t);
+      }
+      break;
+
     case AB_Job_TypeDebitNote:
+      ot=AB_JobSingleDebitNote_GetTransaction(bj);
+      if (ot) {
+	AB_TRANSACTION *t;
+
+	t=AB_Transaction_dup(ot);
+	AB_Transaction_SetStatus(t, tStatus);
+	AB_Transaction_SetType(t, AB_Transaction_TypeDebitNote);
+	AB_ImExporterContext_AddTransfer(ctx, t);
+      }
+      break;
+
+    case AB_Job_TypeInternalTransfer:
+      ot=AB_JobInternalTransfer_GetTransaction(bj);
+      if (ot) {
+	AB_TRANSACTION *t;
+
+	t=AB_Transaction_dup(ot);
+	AB_Transaction_SetStatus(t, tStatus);
+	AB_ImExporterContext_AddTransfer(ctx, t);
+      }
       break;
 
     case AB_Job_TypeCreateStandingOrder:
@@ -1536,7 +1903,9 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 
 	t=AB_Transaction_dup(ot);
 	AB_Transaction_SetFiId(t, aj->fiid);
+	AB_Transaction_SetStatus(t, tStatus);
 	AB_JobCreateStandingOrder_SetTransaction(bj, t);
+        AB_ImExporterContext_AddStandingOrder(ctx, t);
       }
       break;
 
@@ -1547,11 +1916,33 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 
 	t=AB_Transaction_dup(ot);
 	AB_Transaction_SetFiId(t, aj->fiid);
+	AB_Transaction_SetStatus(t, tStatus);
 	AB_JobModifyStandingOrder_SetTransaction(bj, t);
+        AB_ImExporterContext_AddStandingOrder(ctx, t);
       }
       break;
 
     case AB_Job_TypeDeleteStandingOrder:
+      ot=AB_JobDeleteStandingOrder_GetTransaction(bj);
+      if (ot) {
+	AB_TRANSACTION *t;
+
+	t=AB_Transaction_dup(ot);
+        switch(tStatus) {
+	case AB_Transaction_StatusAccepted:
+	  AB_Transaction_SetStatus(t, AB_Transaction_StatusRevoked);
+	  break;
+	case AB_Transaction_StatusPending:
+	  AB_Transaction_SetStatus(t, AB_Transaction_StatusPending);
+	  break;
+	case AB_Transaction_StatusRejected:
+	default:
+	  /* don't modify status */
+	  break;
+        }
+	AB_JobModifyStandingOrder_SetTransaction(bj, t);
+        AB_ImExporterContext_AddStandingOrder(ctx, t);
+      }
       break;
 
     case AB_Job_TypeCreateDatedTransfer:
@@ -1561,7 +1952,9 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 
 	t=AB_Transaction_dup(ot);
 	AB_Transaction_SetFiId(t, aj->fiid);
+	AB_Transaction_SetStatus(t, tStatus);
 	AB_JobCreateDatedTransfer_SetTransaction(bj, t);
+        AB_ImExporterContext_AddDatedTransfer(ctx, t);
       }
       break;
 
@@ -1572,11 +1965,33 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 
 	t=AB_Transaction_dup(ot);
 	AB_Transaction_SetFiId(t, aj->fiid);
+	AB_Transaction_SetStatus(t, tStatus);
 	AB_JobModifyDatedTransfer_SetTransaction(bj, t);
+        AB_ImExporterContext_AddDatedTransfer(ctx, t);
       }
       break;
 
     case AB_Job_TypeDeleteDatedTransfer:
+      ot=AB_JobDeleteDatedTransfer_GetTransaction(bj);
+      if (ot) {
+	AB_TRANSACTION *t;
+
+	t=AB_Transaction_dup(ot);
+        switch(tStatus) {
+	case AB_Transaction_StatusAccepted:
+	  AB_Transaction_SetStatus(t, AB_Transaction_StatusRevoked);
+	  break;
+	case AB_Transaction_StatusPending:
+	  AB_Transaction_SetStatus(t, AB_Transaction_StatusPending);
+	  break;
+	case AB_Transaction_StatusRejected:
+	default:
+	  /* don't modify status */
+	  break;
+        }
+	AB_JobModifyDatedTransfer_SetTransaction(bj, t);
+        AB_ImExporterContext_AddDatedTransfer(ctx, t);
+      }
       break;
 
     default:
@@ -1590,7 +2005,7 @@ int AH_Job_SingleTransfer_Exchange(AH_JOB *j, AB_JOB *bj,
 
   default:
     DBG_NOTICE(AQHBCI_LOGDOMAIN, "Unsupported exchange mode");
-    return AB_ERROR_NOT_SUPPORTED;
+    return GWEN_ERROR_NOT_SUPPORTED;
   } /* switch */
 }
 
