@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile: user_p.h,v $
-                             -------------------
-    cvs         : $Id: user_p.h,v 1.1 2005/08/08 02:24:41 aquamaniac Exp $
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2004 by Martin Preuss
+    copyright   : (C) 2004-2010 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -16,13 +13,29 @@
 #include <aqofxconnect/user.h>
 
 
+typedef struct AO_USER AO_USER;
 struct AO_USER {
-  GWEN_LIST_ELEMENT(AO_USER)
-  AO_BANK *bank;
-  char *userId;
-  char *userName;
+  uint32_t flags;
+
+  char *bankName;
+  char *brokerId;
+  char *org;
+  char *fid;
+  char *serverAddr;
+  char *appId;
+  char *appVer;
+  char *headerVer;
+  char *clientUid;
+
+  char *securityType;
+
+  int httpVMajor;
+  int httpVMinor;
 };
 
+static void GWENHYWFAR_CB AO_User_FreeData(void *bp, void *p);
+
+static void AO_User_ReadDb(AB_USER *u, GWEN_DB_NODE *db);
 
 
 #endif
