@@ -1,7 +1,7 @@
 /***************************************************************************
- $RCSfile: version.h.in,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: version.h.in,v 1.1 2005/08/08 02:24:41 aquamaniac Exp $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
@@ -14,13 +14,13 @@
 #ifndef AQOFXCONNECT_VERSION_H
 #define AQOFXCONNECT_VERSION_H
 
-#define AQOFXCONNECT_VERSION_MAJOR 1
-#define AQOFXCONNECT_VERSION_MINOR 6
-#define AQOFXCONNECT_VERSION_PATCHLEVEL 0
+#define AQOFXCONNECT_VERSION_MAJOR 5
+#define AQOFXCONNECT_VERSION_MINOR 0
+#define AQOFXCONNECT_VERSION_PATCHLEVEL 28
 #define AQOFXCONNECT_VERSION_BUILD 0
 #define AQOFXCONNECT_VERSION_TAG "beta"
-#define AQOFXCONNECT_VERSION_FULL_STRING "1.6.0.0beta"
-#define AQOFXCONNECT_VERSION_STRING "1.6.0"
+#define AQOFXCONNECT_VERSION_FULL_STRING "5.0.28.0beta"
+#define AQOFXCONNECT_VERSION_STRING "5.0.28"
 
 
 
